@@ -5,7 +5,7 @@
 # Author    : Chris Jones
 # Email     : crj341@student.bham.ac.uk
 # Date      : 26/06/2020
-# Version   : 0.3.9
+# Version   : 0.3.11
 
 from operator import contains
 import  numpy                   as      np
@@ -15,6 +15,7 @@ from    scipy                   import  interpolate as  interp
 from    NSFopen                 import  read        as  ns
 import  re
 import  os
+from    sklearn.linear_model    import  LinearRegression
 
 class AFM:
 
@@ -1137,31 +1138,62 @@ class AFM:
         poisson_ratio = 0.5,
         opening_angle = 35,
         plot_hist = False,
-        return_fig = False
+        return_fig = False,
+        start_dist = 1,
+        end_dist = 0,
+        intercept = True
     ):
         '''
         Funciton to calculate the modulus value for each curve using the Sneddon model.
+        This model is appropriate for a conical indenter where the indentation depth is
+        at least 3 times greater than the tip radius. The fitting region on the indentation
+        curve can be selected by setting the 'start_dist' and 'end_dist' as a fraction
+        of the total indent depth, where start distance is the highest indentation depth to use.
+        The intercept option can be set to 'False' to fix the intercept at zero.
         A histogram of calculated values can be plotted by setting the 'plot_hist'
-        input to 'True'
+        input to 'True'.
         '''
         self.modulus = np.zeros(self.z_approach.shape[1])
 
         for curve_number in range(0, self.z_approach.shape[1]):
-            _approach = self.approach[:,curve_number]
-            _z = self.z_approach[:,curve_number]
 
-            force = _approach[np.isfinite(_approach)][0] * 1e-9
-            depth = _z[np.isfinite(_approach)][0] * 1e-9
+            force = self.approach[:,curve_number] * 1e-9
+            depth = self.z_approach[:,curve_number] * 1e-9
 
-            term_1 = 2/np.pi
-            term_2 = 1 / (1 - poisson_ratio ** 2)
-            term_3 = 1 / force
-            term_4 = np.tan(np.deg2rad(opening_angle))
-            term_5 = np.absolute(depth) ** 2
+            max_depth = next(n for n in depth if n == n) * np.array(start_dist)
+            min_depth = next(n for n in depth if n == n) * np.array(end_dist)
 
-            terms = term_1 * term_2 * term_3 * term_4 * term_5
+            force = force[depth < min_depth]
+            depth = depth[depth < min_depth]
+            force = force[depth > max_depth]
+            depth = depth[depth > max_depth]
 
-            self.modulus[curve_number] = (1 / terms) * 1e-6
+            indent_fit = LinearRegression(fit_intercept = intercept)
+
+            x = depth
+            y = force
+
+            x = x[~np.isnan(y)]
+            y = y[~np.isnan(y)]
+
+            y = y[~np.isnan(x)]
+            x = x[~np.isnan(x)]
+            
+            x = x.reshape(-1, 1)
+            y = y.reshape(-1, 1)
+
+
+            indent_fit.fit(x, y)
+
+            c = indent_fit.coef_
+
+            term_1 = np.pi/2
+            term_2 = 1 / np.tan(np.deg2rad(opening_angle))
+            term_3 = (1 - np.array(poisson_ratio) ** 2)
+
+            E = c ** 2 * term_1 * term_2 * term_3  * 1e-6
+
+            self.modulus[curve_number] = E
         
         if plot_hist is True:
             fig = go.Figure(data=[go.Histogram(x = self.modulus)])
